#include "SDL.h"
#include "KG_Util.h"

#include "SDL_image.h"

SDL_Surface *Kokido_SafeLoad(char *name) {
	SDL_Surface *temp, *out;
	/* Loads images, spits out errors, and saves me a ton of work :)*/
	
	temp = IMG_Load(name);
	if ( temp == NULL ) {
		fprintf(stderr, "Unable to load %s: %s\n", "", name, SDL_GetError());
		return(0);
	}
	
	if(temp->format->Amask) {
		out = SDL_DisplayFormatAlpha(temp);
		SDL_SetAlpha(out, SDL_SRCALPHA | SDL_RLEACCEL, 0);
	}
	else {
		out = SDL_DisplayFormat(temp);
	}
	
	SDL_FreeSurface(temp);
	return(out);
}

SDL_Surface *Kokido_CreateDispFmtSurf(Sint32 w, Sint32 h, int use_alpha) {
	SDL_Surface *temp, *out;
	SDL_PixelFormat *fmt;

	fmt = SDL_GetVideoSurface()->format;

	temp=SDL_CreateRGBSurface(0,w,h,fmt->BitsPerPixel,fmt->Rmask,fmt->Gmask,fmt->Bmask,fmt->Amask);
    if(use_alpha) {
		out = SDL_DisplayFormatAlpha(temp);
		SDL_SetAlpha(out, SDL_SRCALPHA | SDL_RLEACCEL, 0);
	}
	else {
		out = SDL_DisplayFormat(temp);
	}
	SDL_FreeSurface(temp);
	return(out);
}

void draw_pixel(Sint32 X, Sint32 Y, Uint32 pixel, SDL_Surface *surf) {
    int bpp = surf->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surf->pixels + Y * surf->pitch + X * bpp;
    switch(bpp) {
    case 1:
        *p = pixel;
        break;
    case 2:
        *(Uint16 *)p = pixel;
        break;
    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } 
		else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;
    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}

Uint32 read_pixel(Sint32 X, Sint32 Y, SDL_Surface *surf) {
    int bpp = surf->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surf->pixels + Y * surf->pitch + X * bpp;
    switch(bpp) {
    case 1:
        return *p;
    case 2:
        return *(Uint16 *)p;
    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;
    case 4:
        return *(Uint32 *)p;
    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}

void draw_vgrad(Sint32 X1, Sint32 Y1, Uint32 c1, 
				Sint32 X2, Sint32 Y2, Uint32 c2, 
				SDL_Surface *surf) {

	static Uint8   *bits, bpp;
	static Uint8 r, g, b, a;
	static Uint8 r2, g2, b2, a2;
	Sint32 X, Y;
	Uint32 pixel;
    
    /* Lock before calling, if necessary */
	if(SDL_MUSTLOCK(surf)) {
		SDL_LockSurface(surf);
	}
	
	for(Y=Y1; Y<Y2; Y++) {
		SDL_GetRGBA(c1, surf->format, &r, &g, &b, &a);		
		SDL_GetRGBA(c2, surf->format, &r2, &g2, &b2, &a2);


		for(X=X1; X<X2; X++) {

			bpp = surf->format->BytesPerPixel;
    
			bits = ((Uint8 *)surf->pixels)+Y*surf->pitch+X*bpp;
			/* Set the pixel */    
			switch(bpp) {        
			case 1:
				*((Uint8 *)(bits)) = (Uint8)pixel;            
				break;        
			case 2:
				*((Uint16 *)(bits)) = (Uint16)pixel;            
				break;
			case 3:  
				/* Format/endian independent */            
				r = (pixel>>surf->format->Rshift)&0xFF;
				g = (pixel>>surf->format->Gshift)&0xFF;
				b = (pixel>>surf->format->Bshift)&0xFF;
				a = (pixel>>surf->format->Ashift)&0xFF;
				*((bits)+surf->format->Rshift/8) = r; 
				*((bits)+surf->format->Gshift/8) = g;
				*((bits)+surf->format->Bshift/8) = b;
				*((bits)+surf->format->Ashift/8) = a;
				break;        
			case 4:
				*((Uint32 *)(bits)) = (Uint32)pixel;            
				break;    
			}

		}
	}
    /* Update the display */    
	if(SDL_MUSTLOCK(surf)) {
		SDL_UnlockSurface(surf);
	}
   return;
}

void ScaledBlit (SDL_Surface *src, SDL_Rect *src_rct, 
				 SDL_Surface *dst, SDL_Rect *dst_rct) {
	Sint32 x, y;
	Sint32 u, v;
	Uint32 pixel;

	if(SDL_MUSTLOCK(src)) {
		SDL_LockSurface(src);
	}
	if(SDL_MUSTLOCK(dst)) {
		SDL_LockSurface(dst);
	}

	for(y = 0; y<dst_rct->h; y++) {
		for(x = 0; x<dst_rct->w; x++) {
			u = src_rct->w * x / dst_rct->w;
			v = src_rct->h * y / dst_rct->h;
			pixel = read_pixel(u + src_rct->x, v + src_rct->y, src);
			draw_pixel(x + dst_rct->x, y + dst_rct->y, pixel, dst);

		}
	}

	if(SDL_MUSTLOCK(src)) {
		SDL_UnlockSurface(src);
	}
	if(SDL_MUSTLOCK(dst)) {
		SDL_UnlockSurface(dst);
	}

}